package iface

import (
	"context"
	"errors"

	path "gitlab.dms3.io/dms3/interface-go-dms3-core/path"

	"gitlab.dms3.io/dms3/interface-go-dms3-core/options"
)

var ErrResolveFailed = errors.New("could not resolve name")

// Dms3NsEntry specifies the interface to Dms3NsEntries
type Dms3NsEntry interface {
	// Name returns Dms3NsEntry name
	Name() string
	// Value returns Dms3NsEntry value
	Value() path.Path
}

type Dms3NsResult struct {
	path.Path
	Err error
}

// NameAPI specifies the interface to DMS3NS.
//
// DMS3NS is a PKI namespace, where names are the hashes of public keys, and the
// private key enables publishing new (signed) values. In both publish and
// resolve, the default name used is the node's own PeerID, which is the hash of
// its public key.
//
// You can use .Key API to list and generate more names and their respective keys.
type NameAPI interface {
	// Publish announces new DMS3NS name
	Publish(ctx context.Context, path path.Path, opts ...options.NamePublishOption) (Dms3NsEntry, error)

	// Resolve attempts to resolve the newest version of the specified name
	Resolve(ctx context.Context, name string, opts ...options.NameResolveOption) (path.Path, error)

	// Search is a version of Resolve which outputs paths as they are discovered,
	// reducing the time to first entry
	//
	// Note: by default, all paths read from the channel are considered unsafe,
	// except the latest (last path in channel read buffer).
	Search(ctx context.Context, name string, opts ...options.NameResolveOption) (<-chan Dms3NsResult, error)
}
